//  aplisttest.cpp
//
//  Chris Nevison
//  February 2, 1998
//
//  This program tests the list class aplist and its iterator class
//  aplistIterator.  It reads a list of names from a file, then sorts
//  them using insertion sort.  It also test the find and the insert
//  operations.

#include <iostream.h>
#include <fstream.h>
#include "apstring.h"
#include "aplist.h"

void DisplayList(const aplist<apstring> & list);
void ReverseDisplayList(const aplist<apstring> & list);
void DisplayListM(const aplist<apstring> & list);
void ReverseDisplayListM(const aplist<apstring> & list);
void ReadList(aplist<apstring> & list, const apstring & filename);
void SortList(aplist<apstring> & list);

int main()
{
  apstring fname;
  aplist<apstring> names;
  char ch;
  
  cout << "Enter file name for names list: ";
  cin >> fname;
  cout << endl;
  
  if(names.IsEmpty()){
    cout << "Initial empty list. " << endl;
  }
  cout << "Here is the empty list displayed:" << endl;
  DisplayList(names); 
  cout << endl << "Done Display." << endl << endl;
  cin >> ch;

  ReadList(names, fname);

  if (names.IsEmpty()){
    cout << "empty list." << endl;
  }
  DisplayList(names); 
  cout << endl;

  names.Append("ryan");
  names.Prepend("matt");
  DisplayList(names);
  cout << endl;
  cin >> ch;

  aplistIterator<apstring> it(names), it2(names);
  it = names.First();
  cout << it.Current() << " ";
  it2 = names.InsertBefore(it,"doug");
  cout << it2.Current() << "  ";
  cout << it.Current() << " ";
  it.Advance();
  cout << it.Current() << endl << endl;

  names.InsertBefore(it, "sheila");
  DisplayList(names);
  cin >> ch;

  while((it.OnList()) && (it.Current() != "joe")){
    it.Advance();
  }
  cout << it.Current() << endl << endl;
  names.Erase(it);
  cout << "Item after joe, which was removed: " << it.Current() << endl << endl;
  DisplayList(names);
  cout << endl;
  cin >> ch;

  while((it.OnList()) && (it.Current() != "ryan")){
    it.Advance();
  }
  names.InsertBefore(it, "evan");
  names.InsertAfter(it, "henry");
  DisplayList(names);
  cout << endl;
  cin >> ch;

  while((it.OnList()) && (it.Current() != "sheila")){
    it.Backup();
  }
  cout << it.Current();
  it.Backup();
  cout << "   " << it.Current();
  it.Backup();
  cout << "   " << it.Current() << endl << endl;
  DisplayList(names);
  cout << endl;
  cin >> ch;

  it.SetCurrent("george");
  cout << "After replacing current with george." << endl;
  DisplayList(names);
  cout << endl;
  cin >> ch;

 
  cout << endl;
  SortList(names);
  DisplayList(names);
  cout << endl;
  cin >> ch;
  aplist<apstring> copy1(names);
  cout << "Display copy 1 in reverse." << endl << endl;
  ReverseDisplayList(copy1);
  cout << endl;
  cin >> ch;

  aplist<apstring> copy2;
  copy2 = names;
  cout << "Display copy 2 with markers." << endl << endl;
  DisplayListM(copy2);
  cout << endl;
  cin >> ch;
  ReverseDisplayListM(names);
  cout << endl;
  
  return 0;
}

void DisplayList(const aplist<apstring> & list)
{
  aplistIterator<apstring> it(list);

  for (it = list.First(); it.OnList(); it.Advance()){
    cout << it.Current() << endl;
  }
}

void ReverseDisplayList(const aplist<apstring> & list)
{
  aplistIterator<apstring> it(list);

  for (it = list.Last(); it.OnList(); it.Backup()){
    cout << it.Current() << endl;
  }
}

void DisplayListM(const aplist<apstring> & list)
{
  aplistIterator<apstring> it(list);

  for (it = list.First(); it != list.EndMarker(); it.Advance()){
    cout << it.Current() << endl;
  }
}

void ReverseDisplayListM(const aplist<apstring> & list)
{
  aplistIterator<apstring> it(list);

  for (it = list.Last(); it != list.FrontMarker(); it.Backup()){
    cout << it.Current() << endl;
  }
}

void ReadList(aplist<apstring> & list, const apstring & filename)
{
  apstring name;
  ifstream in;
  in.open(filename.c_str());
  while (in >> name){
    list.Append(name);
  }
}

void SortList(aplist<apstring> & list)
// post:  list is sorted in increasing (alphabetical) order
{
  //char ch;  // use for debugging or demonstrating the sort
  aplistIterator<apstring> it1(list), it2(list);

  it1 = list.First();
  if (it1.OnList())
    it1.Advance(); 
  while(it1.OnList()){
    // Loop invariant:  all items in the list before the position of it1
    //                  are in sorted order
    it2 = it1;
    it2.Backup();
    while(it2.OnList() && it2.Current() > it1.Current()){
      // Loop invariant:  for any item, n, in the list after the 
      //                  position of it2 and before it1, n > it1()
      it2.Backup();
    }
    if (it2.OnList())
      it2.Advance();
    else
      it2 = list.First();
    list.InsertBefore(it2, it1.Current());
    list.Erase(it1);

  //  DisplayList(list); // use for debugging or demonstrating
  //  cout << endl;      // the operation of the sort
  //  cin >> ch;
  }
}


